@@ -9,6 +9,7 @@ except:
     pyKDTree = spatial.cKDTree
 from meshparty import skeleton_io
 from collections.abc import Iterable
+from .skeleton_utils import resample_path
 
 
 def _metadata_from_dict(
@@ -233,7 +234,7 @@ class StaticSkeleton:
 
         for comp in comps_to_reroot:
             if comp == root_comp:
-                comp_root = new_root
+                comp_root = int(new_root)
             else:
                 comp_root = utils.find_far_points_graph(
                     self.csgraph_binary,
@@ -592,6 +593,20 @@ class Skeleton:
             self._vertices = self._rooted.vertices[self.node_mask]
         return self._vertices
 
+    @vertices.setter
+    def vertices(self, new_vertices):
+        new_vertices = np.atleast_2d(new_vertices)
+        if new_vertices.shape[1] != 3:
+            raise ValueError("New vertices must be 3 dimensional")
+        if len(new_vertices) == self._rooted.n_vertices:
+            self._rooted._vertices = new_vertices
+        elif len(new_vertices) == self.n_vertices:
+            self._rooted._vertices[self.node_mask] = new_vertices
+        else:
+            raise ValueError("New vertices must be the same size as existing vertices")
+        self._reset_derived_properties_rooted()
+        self._reset_derived_properties_filtered(index_changed=False)
+
     @property
     def edges(self):
         if self._edges is None:
@@ -660,7 +675,7 @@ class Skeleton:
     @voxel_scaling.setter
     def voxel_scaling(self, new_scaling):
         self._rooted.voxel_scaling = new_scaling
-        self._reset_derived_properties_filtered()
+        self._reset_derived_properties_filtered(index_changed=False)
 
     #####################
     # Rooted properties #
@@ -712,18 +727,19 @@ class Skeleton:
     # Filtered properties #
     #######################
 
-    def _reset_derived_properties_filtered(self):
+    def _reset_derived_properties_filtered(self, index_changed=True):
         self._vertices = None
         self._edges = None
         self._kdtree = None
         self._pykdtree = None
-        self._branch_points = None
-        self._end_points = None
 
-        self._segments = None
-        self._segment_map = None
-        self._SkeletonIndex = None
-        self._cover_paths = None
+        if index_changed:
+            self._branch_points = None
+            self._end_points = None
+            self._segments = None
+            self._segment_map = None
+            self._SkeletonIndex = None
+            self._cover_paths = None
 
     #########################
     # Geometric quantitites #
@@ -763,7 +779,7 @@ class Skeleton:
         if len(paths) == 0:
             return 0
 
-        if isinstance(paths[0], Iterable):
+        if np.ndim(paths[0]) > 0:
             Ls = []
             for path in paths:
                 Ls.append(self._single_path_length(path))
@@ -851,6 +867,18 @@ class Skeleton:
             self._segments, self._segment_map = self._compute_segments()
         return self._segments
 
+    @property
+    def segments_plus(self):
+        """list : A list of array indices of segments, including the segment parent in the next segmetn"""
+        segs_plus = []
+        for seg in self.segments:
+            parent = self.parent_nodes(seg[-1])
+            if parent > 0:
+                segs_plus.append(self.SkeletonIndex(np.concatenate([seg, [parent]])))
+            else:
+                segs_plus.append(seg)
+        return segs_plus
+
     @property
     def segment_map(self):
         """np.array : N set of of indices between 0 and len(self.segments)-1, denoting
@@ -868,7 +896,7 @@ class Skeleton:
             return_predecessors=True,
         )
         if not np.isinf(d[t_ind]):
-            return self.SkeletonIndex(utils.path_from_predecessors(Ps, t_ind))
+            return self.SkeletonIndex(utils.path_from_predecessors(Ps, t_ind)[::-1])
         else:
             return None
 
@@ -982,7 +1010,7 @@ class Skeleton:
     # Cover Path Functions #
     #####################
 
-    def _compute_cover_paths(self, end_points=None):
+    def _compute_cover_paths(self, end_points=None, include_parent=False):
         """Compute the list of cover paths along the skeleton"""
         cover_paths = []
         seen = np.full(self.n_vertices, False)
@@ -992,7 +1020,12 @@ class Skeleton:
         ep_order = np.argsort(self.distance_to_root[end_points])[::-1]
         for ep in end_points[ep_order]:
             ptr = np.array(self.path_to_root(ep))
-            cover_paths.append(self.SkeletonIndex(ptr[~seen[ptr]]))
+            path = ptr[~seen[ptr]]
+            if include_parent:
+                pn = int(self.parent_nodes(path[-1]))
+                if pn != -1:
+                    path = np.concatenate((path, [pn]))
+            cover_paths.append(self.SkeletonIndex(path))
             seen[ptr] = True
         return cover_paths
 
@@ -1010,7 +1043,7 @@ class Skeleton:
             self._cover_paths = self._compute_cover_paths()
         return self._cover_paths
 
-    def cover_paths_specific(self, end_points):
+    def cover_paths_specific(self, end_points, include_parent=False):
         """Compute nonoverlapping paths from specified endpoints
 
         Parameters
@@ -1024,9 +1057,15 @@ class Skeleton:
                 List of cover paths using the specified end points. Note that this is not sorted in the same order
                 (or necessarily the same length) as specified end points.
         """
-        paths = self._compute_cover_paths(end_points=end_points)
+        paths = self._compute_cover_paths(
+            end_points=end_points, include_parent=include_parent
+        )
         return [p for p in paths if len(p) > 0]
 
+    def cover_paths_with_parent(self):
+        """Compute minimally overlapping paths toward root, including a single parent vertex where the path originates."""
+        return self._compute_cover_paths(include_parent=True)
+
     ####################
     # Export functions #
     ####################
@@ -1047,7 +1086,16 @@ class Skeleton:
         self.voxel_scaling = existing_voxel_scaling
 
     def export_to_swc(
-        self, filename, node_labels=None, radius=None, header=None, xyz_scaling=1000
+        self,
+        filename,
+        node_labels=None,
+        radius=None,
+        header=None,
+        xyz_scaling=1000,
+        resample_spacing=None,
+        interp_kind="linear",
+        tip_length_ratio=0.5,
+        avoid_root=True,
     ):
         """
         Export a skeleton file to an swc file
@@ -1079,4 +1127,73 @@ class Skeleton:
             radius=radius,
             header=header,
             xyz_scaling=xyz_scaling,
+            resample_spacing=resample_spacing,
+            interp_kind=interp_kind,
+            tip_length_ratio=tip_length_ratio,
+            avoid_root=avoid_root,
         )
+
+
+def resample(sk, spacing, kind="linear", tip_length_ratio=0.5, avoid_root=True):
+    """Resample a skeleton's vertices
+
+    Parameters
+    ----------
+    sk : Skeleton
+        Input skeleton file with a skeleton
+    spacing : numeric
+        Desired spacing in nanometers
+    kind : str, optional
+        Type of interpolation to use when resampling. Options follow scipy.interpolate.interp1d. By default "linear"
+    tip_length_ratio : float, optional
+        The ratio of spacing to branch tip length that a branch tip must have in order to be included in the final skeleton
+        for example: spacing is 10 and branch length is 8. do you want to include that final 8 length tip?
+        then perhaps consider a tip_length_ratio of .75, by default 0.25
+
+    Returns
+    -------
+    Skeleton
+        New skeleton with resampled vertices.
+
+    resample_map
+        Array where the ith index corresponds to the ith vertex of the resampled skeleton and the value
+        is the associated index in the original skeleton. To assign vertices, we assign a "domain" to each
+        vertex in the original skeleton that is halfway between the vertex and its neighbors. Resampled
+        vertices that fall within that domain (based on topology and distance-to-root) are then associated
+        with the original vertex.
+    """
+    path_counter = 0
+    branch_d = {}
+    vert_list = []
+    edge_list = []
+    output_map_list = []
+
+    for path in sk.cover_paths:
+        new_verts, new_edges, output_map_path, branch_d = resample_path(
+            path,
+            sk,
+            path_counter,
+            spacing,
+            kind,
+            tip_length_ratio,
+            branch_d,
+            avoid_root,
+        )
+        vert_list.append(new_verts)
+        edge_list.append(new_edges)
+        output_map_list.append(output_map_path)
+        path_counter += len(new_verts)
+
+    new_verts = np.vstack(vert_list)
+    new_edges = np.vstack(edge_list)
+    resample_map = np.concatenate(output_map_list)
+
+    return (
+        Skeleton(
+            new_verts,
+            new_edges,
+            root=branch_d[int(sk.root)],
+            remove_zero_length_edges=False,
+        ),
+        resample_map,
+    )
\ No newline at end of file
